package sys;

import android.app.Activity;
import android.app.KeyguardManager;
import android.content.BroadcastReceiver;
import android.content.Context;
import android.content.Intent;
import android.content.IntentFilter;
import android.content.res.AssetManager;
import android.opengl.GLSurfaceView;
import android.os.Bundle;
import android.util.Log;
import android.view.MotionEvent;
import android.view.WindowManager;

import java.util.concurrent.Executors;
import java.util.concurrent.ScheduledExecutorService;
import java.util.concurrent.ScheduledFuture;
import java.util.concurrent.TimeUnit;

import javax.microedition.khronos.egl.EGL10;
import javax.microedition.khronos.egl.EGLConfig;
import javax.microedition.khronos.egl.EGLDisplay;
import javax.microedition.khronos.opengles.GL10;


/********************
    アクティビティ
 ********************/
public class BaseActivity extends Activity
{
	static {
		System.loadLibrary("native");
    }


	private BaseView		surface_view;				// ビュー
	private BaseRenderer	renderer;					// レンダラー

	private ScheduledExecutorService	executor;		// 定期実行管理
	private ScheduledFuture<?>			future;

	private short[]		touch_status = new short[5*3];	// タッチパネル状態

	public native void		createNative();											// native部開始
	public native int		initNative(int _w, int _h, AssetManager _mgr);			// native部初期化
	public native void		quitNative();											// native部終了
	public native void		pauseNative();											// native部一時停止
	public native boolean	updateNative(short _touch[]);							// native部稼働


	/**********
	    開始
	 **********/
	@Override
	protected void	onCreate(Bundle _savedInstanceState)
	{
		super.onCreate(_savedInstanceState);

		renderer		= new BaseRenderer();							// レンダラー作成
		surface_view	= new BaseView(getApplication(), renderer);		// ビュー作成
		setContentView(surface_view);

		executor = Executors.newSingleThreadScheduledExecutor();		// 定期実行管理

		createNative();													// native部開始
	}

	/**********
	    終了
	 **********/
	@Override
	protected void	onDestroy()
	{
		super.onDestroy();
		quitNative();							// native部終了
		executor.shutdown();
	}

	/**************
	    一時停止
	 **************/
	@Override
	protected void	onPause()
	{
		super.onPause();
		if ( future != null ) {					// 定期実行停止
			future.cancel(false);
			future = null;
			synchronized (renderer)
			{
				pauseNative();					// native部一時停止
			}
		}
		getWindow().clearFlags(WindowManager.LayoutParams.FLAG_KEEP_SCREEN_ON);		// スリープ禁止解除
		surface_view.onPause();
	}

	/**********
	    再開
	 **********/
	@Override
	protected void	onResume()
	{
		super.onResume();
		getWindow().addFlags(WindowManager.LayoutParams.FLAG_KEEP_SCREEN_ON);		// スリープ禁止
		for (int i = 0; i < 5; i++) {			// タッチパネル状態クリア
			touch_status[i*3] = 0;
		}
		if ( ((KeyguardManager)getSystemService(Context.KEYGUARD_SERVICE)).inKeyguardRestrictedInputMode() ) {
			registerReceiver(receiver, new IntentFilter(Intent.ACTION_USER_PRESENT));			// スクリーンロック解除待ち
		}
		else {
			surface_view.onResume();
		}
	}

	private UnLockReceiver	receiver = new UnLockReceiver();		// スクリーンロック解除検知用

	private class UnLockReceiver extends BroadcastReceiver
	{
		@Override
		public void		onReceive(Context context, Intent intent)
		{
			unregisterReceiver(receiver);							// レシーバー登録を解除
			surface_view.onResume();
		}
	}


	/************
	    ビュー
	 ************/
	class BaseView extends GLSurfaceView
	{
		public BaseView(Context _context, BaseRenderer _renderer)
		{
			super(_context);
			setEGLContextClientVersion(2);					// OpenGL ES 2.0使用
			setEGLConfigChooser(new ConfigChooser());
			setRenderer(_renderer);
			setRenderMode(RENDERMODE_WHEN_DIRTY);
		}

		class ConfigChooser implements GLSurfaceView.EGLConfigChooser
		{
			@Override
			public EGLConfig	chooseConfig(EGL10 egl, EGLDisplay display)
			{
				final
				int[]	configAttribs =
				{
					EGL10.EGL_RED_SIZE,		8,
					EGL10.EGL_GREEN_SIZE,	8,
					EGL10.EGL_BLUE_SIZE,	8,
					EGL10.EGL_ALPHA_SIZE,	8,
					EGL10.EGL_NONE
				};

				int[]	num_config = new int[1];

				egl.eglChooseConfig(display, configAttribs, null, 0, num_config);

				EGLConfig[]		configs = new EGLConfig[num_config[0]];

				egl.eglChooseConfig(display, configAttribs, configs, num_config[0], num_config);

				return	configs[0];
			}
		}

		/********************
			タッチイベント
		 ********************/
		public boolean	onTouchEvent(final MotionEvent event)
		{
			if ( future == null ) {
				return	false;
			}

			int		_action = event.getAction();
			int		_index, _id;

			switch ( _action & MotionEvent.ACTION_MASK ) {
			  case MotionEvent.ACTION_DOWN :
			  case MotionEvent.ACTION_POINTER_DOWN :
				_index	= (_action & MotionEvent.ACTION_POINTER_INDEX_MASK) >> MotionEvent.ACTION_POINTER_INDEX_SHIFT;
				_id		= event.getPointerId(_index)*3;
				if ( _id < 5*3 ) {
					touch_status[_id + 1] = (short)event.getX(_index);		// X座標
					touch_status[_id + 2] = (short)event.getY(_index);		// Y座標
					touch_status[_id + 0] = 1;								// タッチ中
				}
				break;

			  case MotionEvent.ACTION_MOVE :
				for (_index = 0; _index < event.getPointerCount(); _index++) {
					_id = event.getPointerId(_index)*3;
					if ( _id < 5*3 ) {
						touch_status[_id + 1] = (short)event.getX(_index);	// X座標
						touch_status[_id + 2] = (short)event.getY(_index);	// Y座標
						touch_status[_id + 0] = 1;							// タッチ中
					}
				}
				break;

			  case MotionEvent.ACTION_UP :
			  case MotionEvent.ACTION_POINTER_UP :
				_index	= (_action & MotionEvent.ACTION_POINTER_INDEX_MASK) >> MotionEvent.ACTION_POINTER_INDEX_SHIFT;
				_id		= event.getPointerId(_index)*3;
				if ( _id < 5*3 ) {
					touch_status[_id + 0] = 0;								// 非タッチ
				}
				break;
			}
			return true;
		}
	}


	/****************
	    レンダラー
	 ****************/
	class BaseRenderer implements GLSurfaceView.Renderer
	{
		public void	onSurfaceCreated(GL10 _gl, EGLConfig _config) {}

		public void	onSurfaceChanged(GL10 gl, int width, int height)
		{
			if ( future == null ) {
				int		period = initNative(width, height, getAssets());		// native部初期化

				future = executor.scheduleAtFixedRate(new Runnable()			// 定期実行開始
				{
					@Override
					public void	run()
					{
						surface_view.requestRender();		// 描画リクエスト
					}
				},
				0, period, TimeUnit.MILLISECONDS);
			}
		}

		public void	onDrawFrame(GL10 gl)
		{
			synchronized (renderer)
			{
				if ( (future != null) && !updateNative(touch_status) ) {		// native部稼働
					finish();
				}
			}
		}
	}
}

/***************** End of File ***************************************************/